@@ -43,7 +43,7 @@
 #' library(sf)
 #' library(mirai)
 #' options(sf_use_s2 = FALSE)
-#' daemons(4, dispatcher = "process")
+#' daemons(4)
 #' ncpath <- system.file("shape/nc.shp", package = "sf")
 #' ncpoly <- sf::st_read(ncpath)
 #' ncpoly <- sf::st_transform(ncpoly, "EPSG:5070")
@@ -123,20 +123,20 @@ par_grid_mirai <-
 
     # parallel worker will take terra class objects
     # if chopin function is used
-    class_vec <-
-      if (pkgname == "chopin") {
-        if (is_extract_at) {
-          "sf"
-        } else {
-          "terra"
-        }
-      } else {
-        pkgname
-      }
+    class_vec <- "sf"
+      # if (pkgname == "chopin") {
+        # if (is_extract_at) {
+        #   "sf"
+        # } else {
+        #   "terra"
+        # }
+      # } else {
+        # pkgname
+      # }
 
     # clean additional arguments
     args_input <- list(...)
-    if (funname == "chopin" && is.null(args_input$.standalone)) {
+    if (pkgname == "chopin" && is.null(args_input$.standalone)) {
       args_input$.standalone <- FALSE
     }
 
@@ -154,7 +154,7 @@ par_grid_mirai <-
     }
 
     # class identity check
-    .check_align_fxy(pkgname, args_input$x, args_input$y)
+    # .check_align_fxy(pkgname, args_input$x, args_input$y)
 
     # Main parallelization
     results <-
@@ -270,7 +270,7 @@ par_grid_mirai <-
 #'  in each higher level feature. Be advised that
 #'  accessing the same file simultaneously with
 #'  multiple processes may result in errors.
-#' @details 
+#' @details
 #' In dynamic dots (`...`), `fun_dist` arguments should include
 #'   x and y where sf/terra class objects or file paths are accepted.
 #'   Hierarchy is interpreted by the `regions_id` argument first.
@@ -340,7 +340,7 @@ par_grid_mirai <-
 #' library(sf)
 #' library(mirai)
 #' options(sf_use_s2 = FALSE)
-#' mirai::daemons(4, dispatcher = "process")
+#' mirai::daemons(4)
 #'
 #' nccnty <- sf::st_read(
 #'   system.file("shape/nc.shp", package = "sf")
@@ -412,15 +412,15 @@ par_hierarchy_mirai <-
     # parallel worker will take terra class objects
     # if chopin function is used
     class_vec <-
-      if (pkgname == "chopin") {
+      # if (pkgname == "chopin") {
         if (is_extract_at) {
           "sf"
         } else {
           "terra"
         }
-      } else {
-        pkgname
-      }
+      # } else {
+      #   pkgname
+      # }
 
     # Track spatraster file path
     args_input$x <- .check_par_spatraster(args_input$x)
@@ -440,7 +440,7 @@ par_hierarchy_mirai <-
     }
 
     # class identity check
-    .check_align_fxy(pkgname, args_input$x, args_input$y)
+    # .check_align_fxy(pkgname, args_input$x, args_input$y)
 
     # Region ID cleaning to get unique high-level IDs
     # what if regions refers to a path string?
@@ -508,7 +508,7 @@ par_hierarchy_mirai <-
                 # subregion header string retrieval
                 region_i <- regions_list[[i]]
                 regions_ids <- vec_regions_id
-                
+
                 # subregion object
                 subregion_in <-
                   regions[startsWith(regions_ids, region_i), ]
@@ -689,7 +689,7 @@ par_hierarchy_mirai <-
 #' library(sf)
 #' library(mirai)
 #' options(sf_use_s2 = FALSE)
-#' mirai::daemons(4, dispatcher = "process")
+#' mirai::daemons(4)
 #'
 #' nccnty <- sf::st_read(
 #'   system.file("shape/nc.shp", package = "sf")
@@ -748,15 +748,15 @@ par_multirasters_mirai <-
     # parallel worker will take terra class objects
     # if chopin function is used
     class_vec <-
-      if (pkgname == "chopin") {
+      # if (pkgname == "chopin") {
         if (is_extract_at) {
           "sf"
         } else {
           "terra"
         }
-      } else {
-        pkgname
-      }
+      # } else {
+      #   pkgname
+      # }
 
     # Unlike other par_* functions, raster paths are not
     # tracked by the function since the raster file paths
